# by DCL manual
=begin
==NumRu::AdvancedDCL::Dateaxis
==={W[ɂ
DateaxisW[͓t`߂̃W[łB
悵悤ƂWɂĂ, PʂƂĐKϊݒ肵ĂȂ΂ȂȂ. ܂W̍, [U[Wn(UC)ɂ0ɑʒu炨Ȃ. , Ƃ11131܂31r[|[gςɊ蓖ĂƂ,EChE̗[l(0, 30)Ǝw肷. 

ȉŃTu[`ɋʂĂ CSIDE, JD0, NDɂĐ. 
* CSIDE - ꏊw肷镶^̈. XɊւĂ'B'ottom, 'T'op, 'U'serwł. YɊւĂ'L'eft, 'R'ight, 'U'serwł. ڂAxisTvQƂ̂. 
* JD0 - UC0ɑꏊ̓t. 5ȏオN, 3,4, 1,21œt킷. Ƃ1990N9119900901ƕ\. 
* ND - JD00ڂƂč悷. Ƃ1990N911990N930܂30`Ƃ JD019900901ƂND29Ƃ΂悢.
 
===p^
p^ p^̌^ ΉDCLpbP[W
:nday (Fixnum) <UCPACK> 
   Dateaxis.xady, Dateaxis.yady gƂ,ƂɖڐƓt`w肷. nday sȂ0, 1, 2, 5, 10, 15̒KȒlI΂(0ȉ̂Ƃ͎`ŖڐƓt͕`Ȃ);zɗ^Ă΂̒lg(l͕s).  DCL ̊֐ 'NUCDAY'߂.
:nchar (Fixnum) <UCPACK>
   Dateaxis.xamn, Dateaxis.yamn gƂ,`w肷. ncharsȂ0, 1, 2, 3, 9̒KȒlI΂( 0 ̂Ƃ͎`Ŗڐƌ͕`Ȃ);zɗ^Ă΂̒lg(l͕s). DCL ̊֐ 'NUCCHR' ߂.
:lower (true or false) <UCPACK>
   Dateaxis.xady, Dateaxis.yady ŕ`2ڈȉɂ邩ǂw肷. true Ȃ珬ɂ; false Ȃ啶̂܂܂ɂ(l false).
:iundef (Fixnum) <UCPACK> 
   lsł邱Ƃϐ(l -999).

܂Ãp^̓\bhƂėp邱ƂłBȂ킿ADateaxis.nday Ńp^ nday ̎QƂA Dateaxis.nday= Ńp^ nday ̕ύXłB

=end

module NumRu
  module AdvancedDCL
    module Dateaxis
      def param_init
	@param = Parameter.new(Common.param)
	
	@param.init("nday", Fixnum, "uciset", "uciget", 0, 0)
	@param.init("nchar", Fixnum, "uciset", "uciget", 0, 0)
	@param.init("lower", TrueClass, "uclset", "uclget", 0, 0)
	@param.init("iundef", Fixnum, "uciset", "uciget", 0, 0)
	
	@axisparam = Axis.param
	@temp_param = @param.dup
	@backup = @param.dup
	
      end
      
      private :param_init
      
#------------------- method for parameters----------------------
      
      def set(key, val)
	@param[key]=val
      end
      
# by kawanabe
=begin
---Dateaxis.set(key, val)
    p^̐ݒsB

    
    * key:p^(String)
    * val:ݒ肷lB^͊ep^QƂ̂ƁB
    gp
        Dateaxis.set("nday", 15)      # nday  15 ɐݒ

=end

      def get(key)
	@param[key]
      end

# by kawanabe
=begin
---Dateaxis.get(key)
    p^QƂB

    
    * key:p^(String)
    gp
        Axis.get("nday")      # nday QƂB

=end

      def xdraw(cside, jd0, nd, x=nil, y=nil, temp=nil)
	if(x.is_a?(Hash))
	  temp = x
	elsif((x.is_a?(NArray))&&(x.rank==2))
	  y = x.shape[1].to_f
	  x = x.shape[0].to_f
	  Basics.checkundef(x, y)
	elsif(Basics.typearray(x) && Basics.typearray(y))
	  Basics.checkundef(x, y)
	elsif(x==nil&&y==nil)
	  Basics.checkundef(1.0, 1.0)
	end
	@param.sync
	@axisparam.sync
	@backup = @temp_param.set(temp, @param, @backup)
	@temp_param.sync
	
	Fig.settrf
	
	DCL.ucxacl(cside, jd0, nd)
	@backup.sync
	@temp_param.clear
      end

# by DCL manual
=begin
---Dateaxis.xdraw(cside, jd0, nd, x=nil, y=nil, temp=nil)
    EENɊւW`. 

    
    * cside (String) : Wꏊw肷.
    * jd0 (Fixnum) : 0ɑꏊ̓tw肷.
    * nd (Fixnum) : 悷w肷.
    * x, y (Array or NArray) : EBhEݒ肷邽߂XAY̔z
    * temp (Hash) : e|p^

    l
    (1)̃\bh UCXACL ĂłB
    (2)Dateaxis.xdraw  Dateaxis.xady, Dateaxis.xamn, Dateaxis.xayr (ȉQ)𑱂ČĂł.

=end 

      def ydraw(cside, jd0, nd, x=nil, y=nil, temp=nil)
	if(x.is_a?(Hash))
	  temp = x
	elsif((x.is_a?(NArray))&&(x.rank==2))
	  y = x.shape[1].to_f
	  x = x.shape[0].to_f
	  Basics.checkundef(x, y)
	elsif(Basics.typearray(x) && Basics.typearray(y))
	  Basics.checkundef(x, y)
	elsif(x==nil&&y==nil)
	  Basics.checkundef(1.0, 1.0)
	end
	@param.sync
	@axisparam.sync
	@backup = @temp_param.set(temp, @param, @backup)
	@temp_param.sync
	
	Fig.settrf
	
	DCL.ucyacl(cside, jd0, nd)
	@backup.sync
	@temp_param.clear
      end

# by DCL manual
=begin
---Dateaxis.ydraw(cside, jd0, nd, x=nil, y=nil, temp=nil)
    EENɊւW`. 

    
    * cside (String) : Wꏊw肷.
    * jd0 (Fixnum) : 0ɑꏊ̓tw肷.
    * nd (Fixnum) : 悷w肷.
    * x, y (Array or NArray) : EBhEݒ肷邽߂XAY̔z
    * temp (Hash) : e|p^

    l
    (1)̃\bh UCYACL ĂłB
    (2)Dateaxis.ydraw  Dateaxis.yady, Dateaxis.yamn, Dateaxis.yayr (ȉQ)𑱂ČĂł.

=end 

      def xady(cside, jd0, nd, x=nil, y=nil, temp=nil)
	if(x.is_a?(Hash))
	  temp = x
	elsif((x.is_a?(NArray))&&(x.rank==2))
	  y = x.shape[1].to_f
	  x = x.shape[0].to_f
	  Basics.checkundef(x, y)
	elsif(Basics.typearray(x) && Basics.typearray(y))
	  Basics.checkundef(x, y)
	elsif(x==nil&&y==nil)
	  Basics.checkundef(1.0, 1.0)
	end
	@param.sync
	@axisparam.sync
	@backup = @temp_param.set(temp, @param, @backup)
	@temp_param.sync
	
	Fig.settrf
	
	DCL.ucxady(cside, jd0, nd)
	@backup.sync
	@temp_param.clear
      end

# by DCL manual
=begin
---Dateaxis.xady(cside, jd0, nd, x=nil, y=nil, temp=nil)
    ɊւW`. 

    
    * cside (String) : Wꏊw肷.
    * jd0 (Fixnum) : 0ɑꏊ̓tw肷.
    * nd (Fixnum) : 悷w肷.
    * x, y (Array or NArray) : EBhEݒ肷邽߂XAY̔z
    * temp (Hash) : e|p^

    l
    (1)̃\bh UCXADY ĂłB
    (2)EBhEw肵ĂȂƂAƂx,yw肷x,yɃEBhEݒ肵A`B
    (3) ƂɖڐƓt`, DCL ̊֐ NUCDAY ʂăp^ nday ߂. nday sȂ0, 1, 2, 5, 10, 15̒KȒlI΂( 0 ȉ̂Ƃ͎`ŖڐƓt͕`Ȃ); zɗ^Ă΂̒lg(l͕s). 
    (4) ڐƓt`߂̏ێ邽߂ ŔzpӂĂ. ̔z̑傫Ɋւ鐧, `ڐƓt͍ő100ł. 
    (5) Dateaxis.xady, Dateaxis.yady ͂ꂼ Axis::Lin.xpaxs, Axis::Lin.ypaxs (islct = 2)Ŏ`Ă; Axis::Lin.xptmk, Axis::Lin.yptmk (islct = 1)Ŗڐ`Ă; Axis::Lin.xplbl, Axis::Lin.yplbl (islct = 1)œ̃x`Ă. , W̍\vf̑, Axis::Lin ̃p^ɂĎQ/ύX邱Ƃł. , icent(zs)  0 Ƃ; irotl(zs)  irotc(zs)QƂėp; lbtwn  false Ƃ. 

=end
    
      def yady(cside, jd0, nd, x=nil, y=nil, temp=nil)
	if(x.is_a?(Hash))
	  temp = x
	elsif((x.is_a?(NArray))&&(x.rank==2))
	  y = x.shape[1].to_f
	  x = x.shape[0].to_f
	  Basics.checkundef(x, y)
	elsif(Basics.typearray(x) && Basics.typearray(y))
	  Basics.checkundef(x, y)
	elsif(x==nil&&y==nil)
	  Basics.checkundef(1.0, 1.0)
	end
	@param.sync
	@axisparam.sync
	@backup = @temp_param.set(temp, @param, @backup)
	@temp_param.sync
	
	Fig.settrf
	
	DCL.ucyady(cside, jd0, nd)
	@backup.sync
	@temp_param.clear
      end
      
# by DCL manual
=begin
---Dateaxis.yady(cside, jd0, nd, x=nil, y=nil, temp=nil)
    ɊւW`. 

    
    * cside (String) : Wꏊw肷.
    * jd0 (Fixnum) : 0ɑꏊ̓tw肷.
    * nd (Fixnum) : 悷w肷.
    * x, y (Array or NArray) : EBhEݒ肷邽߂XAY̔z
    * temp (Hash) : e|p^

    l
    (1)̃\bh UCYADY ĂłB
    (2)EBhEw肵ĂȂƂAƂx,yw肷x,yɃEBhEݒ肵A`B
    (3) ƂɖڐƓt`, DCL ̊֐ NUCDAY ʂăp^ nday ߂. nday sȂ0, 1, 2, 5, 10, 15̒KȒlI΂( 0 ȉ̂Ƃ͎`ŖڐƓt͕`Ȃ); zɗ^Ă΂̒lg(l͕s). 
    (4) ڐƓt`߂̏ێ邽߂ ŔzpӂĂ. ̔z̑傫Ɋւ鐧, `ڐƓt͍ő100ł. 
    (5) Dateaxis.xady, Dateaxis.yady ͂ꂼ Axis::Lin.xpaxs, Axis::Lin.ypaxs (islct = 2)Ŏ`Ă; Axis::Lin.xptmk, Axis::Lin.yptmk (islct = 1)Ŗڐ`Ă; Axis::Lin.xplbl, Axis::Lin.yplbl (islct = 1)œ̃x`Ă. , W̍\vf̑, Axis::Lin ̃p^ɂĎQ/ύX邱Ƃł. , icent(zs)  0 Ƃ; irotl(zs)  irotc(zs)QƂėp; lbtwn  false Ƃ. 

=end
    
      def xamn(cside, jd0, nd, x=nil, y=nil, temp=nil)
	if(x.is_a?(Hash))
	  temp = x
	elsif((x.is_a?(NArray))&&(x.rank==2))
	  y = x.shape[1].to_f
	  x = x.shape[0].to_f
	  Basics.checkundef(x, y)
	elsif(Basics.typearray(x) && Basics.typearray(y))
	  Basics.checkundef(x, y)
	elsif(x==nil&&y==nil)
	  Basics.checkundef(1.0, 1.0)
	end
	@param.sync
	@axisparam.sync
	@backup = @temp_param.set(temp, @param, @backup)
	@temp_param.sync
	
	Fig.settrf
	
	DCL.ucxamn(cside, jd0, nd)
	@backup.sync
	@temp_param.clear
      end

# by DCL manual
=begin
---Dateaxis.xamn(cside, jd0, nd, x=nil, y=nil, temp=nil)
    ɊւW`. 

    
    * cside (String) : Wꏊw肷.
    * jd0 (Fixnum) : 0ɑꏊ̓tw肷.
    * nd (Fixnum) : 悷w肷.
    * x, y (Array or NArray) : EBhEݒ肷邽߂XAY̔z
    * temp (Hash) : e|p^

    l
    (1)̃\bh UCXAMN ĂłB
    (2)EBhEw肵ĂȂƂAƂx,yw肷x,yɃEBhEݒ肵A`B
    (3) `, DCL ̊֐ NUCCHR ʂăp^ nchar ߂. nchar sȂ0, 1, 2, 3, 9 ̒KȒlI΂( 0 ȉ̂Ƃ͎`Ŗڐƌ͕`Ȃ); zɗ^Ă΂̒lg(l͕s). nchar ̒l̂Ƃ͂̐Βl̒̌, 2ڈȍ~ɂ.
    (4) ڐƌ`߂̏ێ邽߂ ŔzpӂĂ. ̔z̑傫Ɋւ鐧, `ڐƌ͍ő 50 ł. 
    (5) Dateaxis.xamn, Dateaxis.yamn ͂ꂼ Axis::Lin.xpaxs, Axis::Lin.ypaxs (islct = 2)Ŏ`Ă; Axis::Lin.xptmk, Axis::Lin.yptmk (islct = 2)Ŗڐ`Ă; Axis::Lin.xplbl, Axis::Lin.yplbl (islct = 2)œ̃x`Ă. , W̍\vf̑, Axis::Lin ̃p^ɂĎQ/ύX邱Ƃł. , icent(zs)  0 Ƃ; irotl(zs)  irotc(zs)QƂėp; lbtwn  true Ƃ. 

=end
    
      def yamn(cside, jd0, nd, x=nil, y=nil, temp=nil)
	if(x.is_a?(Hash))
	  temp = x
	elsif((x.is_a?(NArray))&&(x.rank==2))
	  y = x.shape[1].to_f
	  x = x.shape[0].to_f
	  Basics.checkundef(x, y)
	elsif(Basics.typearray(x) && Basics.typearray(y))
	  Basics.checkundef(x, y)
	elsif(x==nil&&y==nil)
	  Basics.checkundef(1.0, 1.0)
	end
	@param.sync
	@axisparam.sync
	@backup = @temp_param.set(temp, @param, @backup)
	@temp_param.sync
	
	Fig.settrf
	
	DCL.ucyamn(cside, jd0, nd)
	@backup.sync
	@temp_param.clear
      end

# by DCL manual
=begin
---Dateaxis.yamn(cside, jd0, nd, x=nil, y=nil, temp=nil)
    ɊւW`. 

    
    * cside (String) : Wꏊw肷.
    * jd0 (Fixnum) : 0ɑꏊ̓tw肷.
    * nd (Fixnum) : 悷w肷.
    * x, y (Array or NArray) : EBhEݒ肷邽߂XAY̔z
    * temp (Hash) : e|p^

    l
    (1)̃\bh UCYAMN ĂłB
    (2)EBhEw肵ĂȂƂAƂx,yw肷x,yɃEBhEݒ肵A`B
    (3) `, DCL ̊֐ NUCCHR ʂăp^ nchar ߂. nchar sȂ0, 1, 2, 3, 9 ̒KȒlI΂( 0 ȉ̂Ƃ͎`Ŗڐƌ͕`Ȃ); zɗ^Ă΂̒lg(l͕s). nchar ̒l̂Ƃ͂̐Βl̒̌, 2ڈȍ~ɂ.
    (4) ڐƌ`߂̏ێ邽߂ ŔzpӂĂ. ̔z̑傫Ɋւ鐧, `ڐƌ͍ő 50 ł. 
    (5) Dateaxis.xamn, Dateaxis.yamn ͂ꂼ Axis::Lin.xpaxs, Axis::Lin.ypaxs (islct = 2)Ŏ`Ă; Axis::Lin.xptmk, Axis::Lin.yptmk (islct = 2)Ŗڐ`Ă; Axis::Lin.xplbl, Axis::Lin.yplbl (islct = 2)œ̃x`Ă. , W̍\vf̑, Axis::Lin ̃p^ɂĎQ/ύX邱Ƃł. , icent(zs)  0 Ƃ; irotl(zs)  irotc(zs)QƂėp; lbtwn  true Ƃ. 

=end
    
      def xayr(cside, jd0, nd, x=nil, y=nil, temp=nil)
	if(x.is_a?(Hash))
	  temp = x
	elsif((x.is_a?(NArray))&&(x.rank==2))
	  y = x.shape[1].to_f
	  x = x.shape[0].to_f
	  Basics.checkundef(x, y)
	elsif(Basics.typearray(x) && Basics.typearray(y))
	  Basics.checkundef(x, y)
	elsif(x==nil&&y==nil)
	  Basics.checkundef(1.0, 1.0)
	end
	@param.sync
	@axisparam.sync
	@backup = @temp_param.set(temp, @param, @backup)
	@temp_param.sync
	
	Fig.settrf
	
	DCL.ucxayr(cside, jd0, nd)
	@backup.sync
	@temp_param.clear
      end

# by DCL manual
=begin
---Dateaxis.xayr(cside, jd0, nd, x=nil, y=nil, temp=nil)
    NɊւW`. 

    
    * cside (String) : Wꏊw肷.
    * jd0 (Fixnum) : 0ɑꏊ̓tw肷.
    * nd (Fixnum) : 悷w肷.
    * x, y (Array or NArray) : EBhEݒ肷邽߂XAY̔z
    * temp (Hash) : e|p^

    l
    (1)̃\bh UCXAYR ĂłB
    (2)EBhEw肵ĂȂƂAƂx,yw肷x,yɃEBhEݒ肵A`B
    (3) ڐƌ`߂̏ێ邽߂ ŔzpӂĂ. ̔z̑傫Ɋւ鐧, `ڐƌ͍ő 50 ł. 
    (4) Dateaxis.xamn, Dateaxis.yamn ͂ꂼ Axis::Lin.xpaxs, Axis::Lin.ypaxs (islct = 2)Ŏ`Ă; Axis::Lin.xptmk, Axis::Lin.yptmk (islct = 2)Ŗڐ`Ă; Axis::Lin.xplbl, Axis::Lin.yplbl (islct = 2)œ̃x`Ă. , W̍\vf̑, Axis::Lin ̃p^ɂĎQ/ύX邱Ƃł. , icent(zs)  0 Ƃ; irotl(zs)  irotc(zs)QƂėp; lbtwn  true Ƃ. 

=end
    
      def yayr(cside, jd0, nd, x=nil, y=nil, temp=nil)
	if(x.is_a?(Hash))
	  temp = x
	elsif((x.is_a?(NArray))&&(x.rank==2))
	  y = x.shape[1].to_f
	  x = x.shape[0].to_f
	  Basics.checkundef(x, y)
	elsif(Basics.typearray(x) && Basics.typearray(y))
	  Basics.checkundef(x, y)
	elsif(x==nil&&y==nil)
	  Basics.checkundef(1.0, 1.0)
	end
	@param.sync
	@axisparam.sync
	@backup = @temp_param.set(temp, @param, @backup)
	@temp_param.sync
	
	Fig.settrf
	
	DCL.ucyayr(cside, jd0, nd)
	@backup.sync
	@temp_param.clear
      end

# by DCL manual
=begin
---Dateaxis.yayr(cside, jd0, nd, x=nil, y=nil, temp=nil)
    NɊւW`. 

    
    * cside (String) : Wꏊw肷.
    * jd0 (Fixnum) : 0ɑꏊ̓tw肷.
    * nd (Fixnum) : 悷w肷.
    * x, y (Array or NArray) : EBhEݒ肷邽߂XAY̔z
    * temp (Hash) : e|p^

    l
    (1)̃\bh UCYAYR ĂłB
    (2)EBhEw肵ĂȂƂAƂx,yw肷x,yɃEBhEݒ肵A`B
    (3) ڐƌ`߂̏ێ邽߂ ŔzpӂĂ. ̔z̑傫Ɋւ鐧, `ڐƌ͍ő 50 ł. 
    (4) Dateaxis.xamn, Dateaxis.yamn ͂ꂼ Axis::Lin.xpaxs, Axis::Lin.ypaxs (islct = 2)Ŏ`Ă; Axis::Lin.xptmk, Axis::Lin.yptmk (islct = 2)Ŗڐ`Ă; Axis::Lin.xplbl, Axis::Lin.yplbl (islct = 2)œ̃x`Ă. , W̍\vf̑, Axis::Lin ̃p^ɂĎQ/ύX邱Ƃł. , icent(zs)  0 Ƃ; irotl(zs)  irotc(zs)QƂėp; lbtwn  true Ƃ. 

=end

      module_function :param_init
      module_function :xdraw
      module_function :ydraw
      module_function :xady
      module_function :yady
      module_function :set
      module_function :get
      module_function :xamn
      module_function :yamn
      module_function :xayr
      module_function :yayr
      
      param_init
      
      @param.keys.each{|i|
	eval "def #{i}; @param['#{i}']; end"
	eval "def #{i}=(val); set('#{i}', val); end"
      }
      
      @param.keys.each{|i|
	eval "module_function :#{i}"
	eval "module_function :#{i}="
      }
    end
  end
end
